# Supporting R script for: Brierley, L., Pedersen, A.B.P., and Woolhouse, M.E.J. Tissue Tropism and Transmission Ecology Predict Virulence of Human RNA Viruses.
# Prepared by L Brierley, 23/6/19

#############################
# Data and analytical setup #
#############################

# Read in required packages
library(rpart)				    
library(rpart.plot)	
library(RColorBrewer)
library(caret)
library(ROCR)
library(randomForest)
library(missForest)
library(pdp)
library(pbapply)

# Read in and process data, selecting only those viruses with reliable virulence data for includsion in analysis
vir <- read.csv("Virulence_data_Brierley_et_al_21_6_19.csv", sep=",", header=T)
vir <- subset(vir, Included==1)

# Remove 'Unassigned' as a family and rearrange by genome type
vir$Family <- droplevels(vir$Family)
vir$Family <- factor(vir$Family, levels=c("Arenaviridae","Bornaviridae","Filoviridae","Hantaviridae","Nairoviridae","Orthomyxoviridae","Paramyxoviridae","Peribunyaviridae","Phenuiviridae","Pneumoviridae","Rhabdoviridae","Astroviridae","Caliciviridae","Coronaviridae","Flaviviridae","Hepeviridae","Picornaviridae","Togaviridae","Picobirnaviridae","Reoviridae","Retroviridae"), labels=c("Arena-","Borna-","Filo-","Hanta-","Nairo-","Orthomyxo-","Paramyxo-","Peribunya-","Phenui-","Pneumo-","Rhabdo-","Astro-","Calici-","Corona-","Flavi-","Hepe-","Picorna-","Toga-","Picobirna-","Reo-","Retro-"))

# Convert 0/1 columns to factors
factor_columns <- c(names(vir[,grepl( "^Tr\\.|^Fatal\\.|^Tp\\.|^Host\\." , names(vir))]))
vir[factor_columns] <- lapply(vir[factor_columns], factor)

# Create True Skill Statistic function
trueskillstat <- function(matrix){
  as.numeric(matrix$byClass[1] + matrix$byClass[2] - 1)
}

# Specify predictor variables to use in models as a formula
formula_used <- formula(Virulence ~ Family+Genome+
                            Tr.level+
                            Tr.primary+
                            Tr.direct.contact+Tr.faecal.oral+Tr.respiratory+Tr.vector+Tr.multiple+Tr.food.borne+Tr.vertical+
                            Tp.primary+
                            Tp.vascular+Tp.circulatory+Tp.gastrointestinal+Tp.hepatic+Tp.neural+Tp.respiratory+Tp.cardiac+Tp.joints+Tp.renal+Tp.reproductive+Tp.sensory+Tp.skin+Tp.muscular+Tp.endocrine+Tp.multiple+
                            Host.range+Host.human.only+Host.nh.primates+Host.other.mammal+Host.bird)

################################
# Classification tree analysis #
################################

# Set random seed for reproducible training/test partition
set.seed(1353)

# Establish training and test partition based on severity and taxonomy at 3:1 ratio (if < 4 viruses exist for a given virulence-genus combination, they default to the training set)
training_row_nos = suppressWarnings(createDataPartition(paste(vir$Virulence, vir$Genus), p = 0.75, list = FALSE))
vir_train <- vir[training_row_nos,]
vir_test <- vir[-training_row_nos,]
# Store test set virus species names
vir_test_names <- vir_test$Species

# Build single classification stree and store complexity parameter table
class_tree <- rpart(formula_used, data=vir_train, method="class", minsplit=10)
cptabledf <- as.data.frame(class_tree$cptable)

# Construct and store optimal branching size for 1000 repeats of a single classification tree
nsplitopt <- vector(mode = "integer", length = 1000)
for (i in 1:length(nsplitopt)) {
  cp <- rpart(formula_used, data=vir_train, minsplit=10)$cptable
  nsplitopt[i] <- cp[which.min(cp[,"xerror"]), "nsplit"]
}

# Extract consensus branching size
consensus_size <- unique(nsplitopt)[which.max(tabulate(match(nsplitopt, unique(nsplitopt))))]

# Prune single tree to the consensus branching size
class_tree_pruned <- prune(class_tree, cp = cptabledf$CP[which(cptabledf$nsplit==consensus_size)])

# Visualise the tree
prp(class_tree_pruned, extra=2, type=3, faclen=2, varlen=0, tweak=1.07, fallen.leaves=TRUE, ycompress=FALSE, yesno=F, box.col=c("gray80", "gray50")[class_tree_pruned$frame$yval])

# Prediction diagnostics and accuracy statistics for training set
predict_tree_class_train <- predict(class_tree_pruned, type="class")
print(matrix_tree_train <- confusionMatrix(predict_tree_class_train, vir_train$Virulence, positive="severe"))

# Prediction diagnostics and accuracy statistics for test set
predict_tree_class_test <- predict(class_tree_pruned, newdata=vir_test, type="class")
print(matrix_tree_test <- confusionMatrix(predict_tree_class_test, vir_test$Virulence, positive="severe")) 

# View predictions from the classification tree for individual virus species in test set
data.frame(Species = as.character(vir_test$Species), Virulence = as.character(vir_test$Virulence), Predicted = as.character(predict_tree_class_test))

# Calculate ROC curve for classification tree
predict_tree_prob_test <- predict(class_tree_pruned, newdata=vir_test, type="prob")
roc_pred_tree <- prediction(predict_tree_prob_test[,2], as.numeric(vir_test$Virulence)-1)
# Calculate AUROC for classification tree
print(AUC_class_tree <- performance(roc_pred_tree, measure="auc")@y.values)
# Calculate TSS for classification tree
trueskillstat(matrix_tree_test)

##########################
# Random forest analysis #
##########################

# Set number of alternative training/test partitions to use
n_sens <- 200

# Set random seed for reproducible training/test partitions
set.seed(1455)

# Establish training/test partitions as a list
training_row_nos_sens = suppressWarnings(createDataPartition(paste(vir$Sev2, vir$Genus), p = 0.75, list = TRUE, times=n_sens))

# Run random forest analysis and return variable importance, partial dependence, predictive diagnostics, test set predictions, ROC curve and AUC as list of lists
output_list <- pblapply(training_row_nos_sens, function(x) {
  
  assign("vir_train", vir[x,], envir = .GlobalEnv)
  assign("vir_test", vir[-x,], envir = .GlobalEnv)
  vir_test_names <- vir_test$Species
  
  source("Brierley_et_al_random_forest_23_6_19.R")
  
  list(varimp=varimp,
       df_PD_prob=df_PD_prob,
       matrix_forest_test=matrix_forest_test,
       predict_forest_test=data.frame(Species = as.character(vir_test_names), Virulence = as.character(vir_test$Virulence), Predicted = as.character(predict_forest_class_test), ProbabilitySevere = predict_forest_prob_test[,2]),
       roc_pred_forest=roc_pred_forest,
       AUC_class_forest=AUC_class_forest)
})

#### PLOT ROC CURVES AND CALCULATE ACCURACY ETC

# Plot ROC curves
plot(performance(roc_pred_tree, measure="tpr", x.measure="fpr"), lwd=2.5, cex.axis=2)
plot(performance(roc_pred_forest, measure="tpr", x.measure="fpr"), add=TRUE, lwd=2.5, col = "red")
abline(a=0, b= 1)
legend(0.65,0.2, 
       c("Classification tree","Random forest"),
       lty=c(1,1),
       lwd=c(2.5,2.5),col=c("black","red")) 